"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformEIP1193Provider = transformEIP1193Provider;
const viem_1 = require("viem");
const accounts_1 = require("viem/accounts");
const AccountFactory_js_1 = require("./abis/AccountFactory.js");
const abstractClient_js_1 = require("./abstractClient.js");
const constants_js_1 = require("./constants.js");
const eip712_js_1 = require("./eip712.js");
const utils_js_1 = require("./utils.js");
async function getAgwAddressFromInitialSigner(chain, transport, signer) {
    const publicClient = (0, viem_1.createPublicClient)({
        chain,
        transport,
    });
    return await (0, utils_js_1.getSmartAccountAddressFromInitialSigner)(signer, publicClient);
}
async function getAgwSigner(provider, method = 'eth_accounts') {
    const accounts = await provider.request({ method });
    return accounts?.[0];
}
async function getAgwTypedSignature(provider, account, signer, messageHash) {
    const chainId = await provider.request({ method: 'eth_chainId' });
    const typedData = (0, viem_1.serializeTypedData)({
        domain: {
            name: 'AbstractGlobalWallet',
            version: '1.0.0',
            chainId: (0, viem_1.fromHex)(chainId, 'bigint'),
            verifyingContract: account,
        },
        types: {
            EIP712Domain: [
                { name: 'name', type: 'string' },
                { name: 'version', type: 'string' },
                { name: 'chainId', type: 'uint256' },
                { name: 'verifyingContract', type: 'address' },
            ],
            ClaveMessage: [{ name: 'signedHash', type: 'bytes32' }],
        },
        message: {
            signedHash: messageHash,
        },
        primaryType: 'ClaveMessage',
    });
    const rawSignature = await provider.request({
        method: 'eth_signTypedData_v4',
        params: [signer, typedData],
    });
    const signature = (0, viem_1.encodeAbiParameters)((0, viem_1.parseAbiParameters)(['bytes', 'address']), [rawSignature, constants_js_1.VALIDATOR_ADDRESS]);
    const addressBytes = (0, viem_1.toBytes)(signer);
    const salt = (0, viem_1.keccak256)(addressBytes);
    return (0, viem_1.serializeErc6492Signature)({
        address: constants_js_1.SMART_ACCOUNT_FACTORY_ADDRESS,
        data: (0, viem_1.encodeFunctionData)({
            abi: AccountFactory_js_1.default,
            functionName: 'deployAccount',
            args: [
                salt,
                (0, utils_js_1.getInitializerCalldata)(signer, constants_js_1.VALIDATOR_ADDRESS, {
                    target: viem_1.zeroAddress,
                    allowFailure: false,
                    callData: '0x',
                    value: 0n,
                }),
            ],
        }),
        signature,
    });
}
function transformEIP1193Provider(options) {
    const { provider, chain, transport: overrideTransport, isPrivyCrossApp = false, } = options;
    const transport = overrideTransport ?? (0, viem_1.custom)(provider);
    const handler = async (e) => {
        const { method, params } = e;
        switch (method) {
            case 'eth_requestAccounts': {
                const signer = await getAgwSigner(provider, method);
                if (!signer) {
                    return [];
                }
                const smartAccount = await getAgwAddressFromInitialSigner(chain, transport, signer);
                return [smartAccount, signer];
            }
            case 'eth_accounts': {
                const signer = await getAgwSigner(provider);
                if (!signer) {
                    return [];
                }
                const smartAccount = await getAgwAddressFromInitialSigner(chain, transport, signer);
                return [smartAccount, signer];
            }
            case 'eth_signTypedData_v4': {
                const signer = await getAgwSigner(provider);
                if (!signer) {
                    throw new Error('Account not found');
                }
                if (params[0] === signer) {
                    return provider.request(e);
                }
                const parsedTypedData = JSON.parse(params[1]);
                if (parsedTypedData?.message &&
                    parsedTypedData?.domain?.name === 'zkSync' &&
                    (0, eip712_js_1.isEIP712Transaction)(parsedTypedData.message)) {
                    return provider.request({
                        method: 'eth_signTypedData_v4',
                        params: [signer, params[1]],
                    });
                }
                return await getAgwTypedSignature(provider, params[0], signer, (0, viem_1.hashTypedData)(parsedTypedData));
            }
            case 'personal_sign': {
                const signer = await getAgwSigner(provider);
                if (!signer) {
                    throw new Error('Account not found');
                }
                if (params[1] === signer) {
                    return provider.request(e);
                }
                return await getAgwTypedSignature(provider, params[1], signer, (0, viem_1.hashMessage)({
                    raw: params[0],
                }));
            }
            case 'eth_signTransaction':
            case 'eth_sendTransaction': {
                const account = await getAgwSigner(provider);
                if (!account) {
                    throw new Error('Account not found');
                }
                const transaction = params[0];
                if (transaction.from === account) {
                    return await provider.request(e);
                }
                const wallet = (0, viem_1.createWalletClient)({
                    account,
                    transport,
                });
                const signer = (0, accounts_1.toAccount)({
                    address: account,
                    signMessage: wallet.signMessage,
                    signTransaction: wallet.signTransaction,
                    signTypedData: wallet.signTypedData,
                });
                const abstractClient = await (0, abstractClient_js_1.createAbstractClient)({
                    chain,
                    signer,
                    transport,
                    isPrivyCrossApp,
                });
                if (transaction.eip712Meta && transaction.eip712Meta.paymasterParams) {
                    transaction.paymaster =
                        transaction.eip712Meta.paymasterParams.paymaster;
                    transaction.paymasterInput = (0, viem_1.toHex)(transaction.eip712Meta.paymasterParams.paymasterInput);
                }
                if (method === 'eth_signTransaction') {
                    return (await abstractClient.signTransaction(transaction));
                }
                else if (method === 'eth_sendTransaction') {
                    return await abstractClient.sendTransaction(transaction);
                }
                throw new Error('Should not have reached this point');
            }
            default: {
                return await provider.request(e);
            }
        }
    };
    return {
        ...provider,
        on: provider.on,
        removeListener: provider.removeListener,
        request: handler,
    };
}
//# sourceMappingURL=transformEIP1193Provider.js.map